@@ -332,7 +332,9 @@ static bool function_call_walker(Node *node, void *context)
 
 			if (ctx->pg_terminate_backend_pid == 0 && strcmp("pg_terminate_backend", fname) == 0)
 			{
-				if (list_length(fcall->args) == 1)
+				/* PostgreSQL 14 or after accept two arguments. */
+				if (list_length(fcall->args) == 1 ||
+					list_length(fcall->args) == 2)
 				{
 					Node *arg = linitial(fcall->args);
 					if (IsA(arg, A_Const) &&
@@ -1200,6 +1202,12 @@ select_table_walker(Node *node, void *context)
 		}
 	}
 
+	/* Skip Data-Modifying Statements in SELECT. */
+	else if (IsA(node, InsertStmt) || IsA(node, DeleteStmt) || IsA(node, UpdateStmt))
+	{
+		return false;
+	}
+
 	return raw_expression_tree_walker(node, select_table_walker, context);
 }
 
